@@ -1,4 +1,4 @@
-// Copyright 2013 Daniel Parker
+// Copyright 2013-2023 Daniel Parker
 // Distributed under the Boost license, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
@@ -160,7 +160,7 @@ bool is_base10(const CharT* s, std::size_t length)
 }
 
 template <class T, class CharT>
-typename std::enable_if<type_traits::integer_limits<T>::is_specialized && !type_traits::integer_limits<T>::is_signed,to_integer_result<T,CharT>>::type
+typename std::enable_if<extension_traits::integer_limits<T>::is_specialized && !extension_traits::integer_limits<T>::is_signed,to_integer_result<T,CharT>>::type
 to_integer_decimal(const CharT* s, std::size_t length, T& n)
 {
     n = 0;
@@ -195,7 +195,7 @@ to_integer_decimal(const CharT* s, std::size_t length, T& n)
             }
             case integer_chars_state::decimal:
             {
-                static constexpr T max_value = (type_traits::integer_limits<T>::max)();
+                static constexpr T max_value = (extension_traits::integer_limits<T>::max)();
                 static constexpr T max_value_div_10 = max_value / 10;
                 for (; s < end; ++s)
                 {
@@ -230,7 +230,7 @@ to_integer_decimal(const CharT* s, std::size_t length, T& n)
 }
 
 template <class T, class CharT>
-typename std::enable_if<type_traits::integer_limits<T>::is_specialized && type_traits::integer_limits<T>::is_signed,to_integer_result<T,CharT>>::type
+typename std::enable_if<extension_traits::integer_limits<T>::is_specialized && extension_traits::integer_limits<T>::is_signed,to_integer_result<T,CharT>>::type
 to_integer_decimal(const CharT* s, std::size_t length, T& n)
 {
     n = 0;
@@ -247,7 +247,7 @@ to_integer_decimal(const CharT* s, std::size_t length, T& n)
         --length;
     }
 
-    using U = typename type_traits::make_unsigned<T>::type;
+    using U = typename extension_traits::make_unsigned<T>::type;
 
     U u;
     auto ru = to_integer_decimal(s, length, u);
@@ -257,7 +257,7 @@ to_integer_decimal(const CharT* s, std::size_t length, T& n)
     }
     if (is_negative)
     {
-        if (u > static_cast<U>(-((type_traits::integer_limits<T>::lowest)()+T(1))) + U(1))
+        if (u > static_cast<U>(-((extension_traits::integer_limits<T>::lowest)()+T(1))) + U(1))
         {
             return to_integer_result<T,CharT>(ru.ptr, to_integer_errc::overflow);
         }
@@ -269,7 +269,7 @@ to_integer_decimal(const CharT* s, std::size_t length, T& n)
     }
     else
     {
-        if (u > static_cast<U>((type_traits::integer_limits<T>::max)()))
+        if (u > static_cast<U>((extension_traits::integer_limits<T>::max)()))
         {
             return to_integer_result<T,CharT>(ru.ptr, to_integer_errc::overflow);
         }
@@ -282,7 +282,7 @@ to_integer_decimal(const CharT* s, std::size_t length, T& n)
 }
 
 template <class T, class CharT>
-typename std::enable_if<type_traits::integer_limits<T>::is_specialized && !type_traits::integer_limits<T>::is_signed,to_integer_result<T,CharT>>::type
+typename std::enable_if<extension_traits::integer_limits<T>::is_specialized && !extension_traits::integer_limits<T>::is_signed,to_integer_result<T,CharT>>::type
 to_integer_base16(const CharT* s, std::size_t length, T& n)
 {
     n = 0;
@@ -319,7 +319,7 @@ to_integer_base16(const CharT* s, std::size_t length, T& n)
             }
             case integer_chars_state::base16:
             {
-                static constexpr T max_value = (type_traits::integer_limits<T>::max)();
+                static constexpr T max_value = (extension_traits::integer_limits<T>::max)();
                 static constexpr T max_value_div_16 = max_value / 16;
                 for (; s < end; ++s)
                 {
@@ -328,13 +328,13 @@ to_integer_base16(const CharT* s, std::size_t length, T& n)
                     switch(*s)
                     {
                         case '0':case '1':case '2':case '3':case '4':case '5':case '6':case '7':case '8': case '9':
-                            x = c - '0';
+                            x = static_cast<T>(c - '0');
                             break;
                         case 'a':case 'b':case 'c':case 'd':case 'e':case 'f':
-                            x = c - ('a' - 10);
+                            x = static_cast<T>(c - ('a' - 10));
                             break;
                         case 'A':case 'B':case 'C':case 'D':case 'E':case 'F':
-                            x = c - ('A' - 10);
+                            x = static_cast<T>(c - ('A' - 10));
                             break;
                         default:
                             return to_integer_result<T,CharT>(s, to_integer_errc::invalid_digit);
@@ -361,7 +361,7 @@ to_integer_base16(const CharT* s, std::size_t length, T& n)
 }
 
 template <class T, class CharT>
-typename std::enable_if<type_traits::integer_limits<T>::is_specialized && type_traits::integer_limits<T>::is_signed,to_integer_result<T,CharT>>::type
+typename std::enable_if<extension_traits::integer_limits<T>::is_specialized && extension_traits::integer_limits<T>::is_signed,to_integer_result<T,CharT>>::type
 to_integer_base16(const CharT* s, std::size_t length, T& n)
 {
     n = 0;
@@ -378,7 +378,7 @@ to_integer_base16(const CharT* s, std::size_t length, T& n)
         --length;
     }
 
-    using U = typename type_traits::make_unsigned<T>::type;
+    using U = typename extension_traits::make_unsigned<T>::type;
 
     U u;
     auto ru = to_integer_base16(s, length, u);
@@ -388,7 +388,7 @@ to_integer_base16(const CharT* s, std::size_t length, T& n)
     }
     if (is_negative)
     {
-        if (u > static_cast<U>(-((type_traits::integer_limits<T>::lowest)()+T(1))) + U(1))
+        if (u > static_cast<U>(-((extension_traits::integer_limits<T>::lowest)()+T(1))) + U(1))
         {
             return to_integer_result<T,CharT>(ru.ptr, to_integer_errc::overflow);
         }
@@ -400,7 +400,7 @@ to_integer_base16(const CharT* s, std::size_t length, T& n)
     }
     else
     {
-        if (u > static_cast<U>((type_traits::integer_limits<T>::max)()))
+        if (u > static_cast<U>((extension_traits::integer_limits<T>::max)()))
         {
             return to_integer_result<T,CharT>(ru.ptr, to_integer_errc::overflow);
         }
@@ -413,7 +413,7 @@ to_integer_base16(const CharT* s, std::size_t length, T& n)
 }
 
 template <class T, class CharT>
-typename std::enable_if<type_traits::integer_limits<T>::is_specialized && !type_traits::integer_limits<T>::is_signed,to_integer_result<T,CharT>>::type
+typename std::enable_if<extension_traits::integer_limits<T>::is_specialized && !extension_traits::integer_limits<T>::is_signed,to_integer_result<T,CharT>>::type
 to_integer(const CharT* s, std::size_t length, T& n)
 {
     n = 0;
@@ -469,7 +469,7 @@ to_integer(const CharT* s, std::size_t length, T& n)
             }
             case integer_chars_state::binary:
             {
-                static constexpr T max_value = (type_traits::integer_limits<T>::max)();
+                static constexpr T max_value = (extension_traits::integer_limits<T>::max)();
                 static constexpr T max_value_div_2 = max_value / 2;
                 for (; s < end; ++s)
                 {
@@ -497,7 +497,7 @@ to_integer(const CharT* s, std::size_t length, T& n)
             }
             case integer_chars_state::octal:
             {
-                static constexpr T max_value = (type_traits::integer_limits<T>::max)();
+                static constexpr T max_value = (extension_traits::integer_limits<T>::max)();
                 static constexpr T max_value_div_8 = max_value / 8;
                 for (; s < end; ++s)
                 {
@@ -525,7 +525,7 @@ to_integer(const CharT* s, std::size_t length, T& n)
             }
             case integer_chars_state::decimal:
             {
-                static constexpr T max_value = (type_traits::integer_limits<T>::max)();
+                static constexpr T max_value = (extension_traits::integer_limits<T>::max)();
                 static constexpr T max_value_div_10 = max_value / 10;
                 for (; s < end; ++s)
                 {
@@ -553,7 +553,7 @@ to_integer(const CharT* s, std::size_t length, T& n)
             }
             case integer_chars_state::base16:
             {
-                static constexpr T max_value = (type_traits::integer_limits<T>::max)();
+                static constexpr T max_value = (extension_traits::integer_limits<T>::max)();
                 static constexpr T max_value_div_16 = max_value / 16;
                 for (; s < end; ++s)
                 {
@@ -596,7 +596,7 @@ to_integer(const CharT* s, std::size_t length, T& n)
 }
 
 template <class T, class CharT>
-typename std::enable_if<type_traits::integer_limits<T>::is_specialized && type_traits::integer_limits<T>::is_signed,to_integer_result<T,CharT>>::type
+typename std::enable_if<extension_traits::integer_limits<T>::is_specialized && extension_traits::integer_limits<T>::is_signed,to_integer_result<T,CharT>>::type
 to_integer(const CharT* s, std::size_t length, T& n)
 {
     n = 0;
@@ -613,7 +613,7 @@ to_integer(const CharT* s, std::size_t length, T& n)
         --length;
     }
 
-    using U = typename type_traits::make_unsigned<T>::type;
+    using U = typename extension_traits::make_unsigned<T>::type;
 
     U u;
     auto ru = to_integer(s, length, u);
@@ -623,7 +623,7 @@ to_integer(const CharT* s, std::size_t length, T& n)
     }
     if (is_negative)
     {
-        if (u > static_cast<U>(-((type_traits::integer_limits<T>::lowest)()+T(1))) + U(1))
+        if (u > static_cast<U>(-((extension_traits::integer_limits<T>::lowest)()+T(1))) + U(1))
         {
             return to_integer_result<T,CharT>(ru.ptr, to_integer_errc::overflow);
         }
@@ -635,7 +635,7 @@ to_integer(const CharT* s, std::size_t length, T& n)
     }
     else
     {
-        if (u > static_cast<U>((type_traits::integer_limits<T>::max)()))
+        if (u > static_cast<U>((extension_traits::integer_limits<T>::max)()))
         {
             return to_integer_result<T,CharT>(ru.ptr, to_integer_errc::overflow);
         }
@@ -648,7 +648,7 @@ to_integer(const CharT* s, std::size_t length, T& n)
 }
 
 template <class T, class CharT>
-typename std::enable_if<type_traits::integer_limits<T>::is_specialized,to_integer_result<T,CharT>>::type
+typename std::enable_if<extension_traits::integer_limits<T>::is_specialized,to_integer_result<T,CharT>>::type
 to_integer(const CharT* s, T& n)
 {
     return to_integer<T,CharT>(s, std::char_traits<CharT>::length(s), n);
@@ -662,17 +662,17 @@ to_integer(const CharT* s, T& n)
 // - digit1-digits
 
 template <class T, class CharT>
-typename std::enable_if<type_traits::integer_limits<T>::is_specialized && !type_traits::integer_limits<T>::is_signed,to_integer_result<T,CharT>>::type
+typename std::enable_if<extension_traits::integer_limits<T>::is_specialized && !extension_traits::integer_limits<T>::is_signed,to_integer_result<T,CharT>>::type
 to_integer_unchecked(const CharT* s, std::size_t length, T& n)
 {
-    static_assert(type_traits::integer_limits<T>::is_specialized, "Integer type not specialized");
+    static_assert(extension_traits::integer_limits<T>::is_specialized, "Integer type not specialized");
     JSONCONS_ASSERT(length > 0);
 
     n = 0;
     const CharT* end = s + length; 
     if (*s == '-')
     {
-        static constexpr T min_value = (type_traits::integer_limits<T>::lowest)();
+        static constexpr T min_value = (extension_traits::integer_limits<T>::lowest)();
         static constexpr T min_value_div_10 = min_value / 10;
         ++s;
         for (; s < end; ++s)
@@ -693,7 +693,7 @@ to_integer_unchecked(const CharT* s, std::size_t length, T& n)
     }
     else
     {
-        static constexpr T max_value = (type_traits::integer_limits<T>::max)();
+        static constexpr T max_value = (extension_traits::integer_limits<T>::max)();
         static constexpr T max_value_div_10 = max_value / 10;
         for (; s < end; ++s)
         {
@@ -723,10 +723,10 @@ to_integer_unchecked(const CharT* s, std::size_t length, T& n)
 // - digit1-digits
 
 template <class T, class CharT>
-typename std::enable_if<type_traits::integer_limits<T>::is_specialized && type_traits::integer_limits<T>::is_signed,to_integer_result<T,CharT>>::type
+typename std::enable_if<extension_traits::integer_limits<T>::is_specialized && extension_traits::integer_limits<T>::is_signed,to_integer_result<T,CharT>>::type
 to_integer_unchecked(const CharT* s, std::size_t length, T& n)
 {
-    static_assert(type_traits::integer_limits<T>::is_specialized, "Integer type not specialized");
+    static_assert(extension_traits::integer_limits<T>::is_specialized, "Integer type not specialized");
     JSONCONS_ASSERT(length > 0);
 
     n = 0;
@@ -734,7 +734,7 @@ to_integer_unchecked(const CharT* s, std::size_t length, T& n)
     const CharT* end = s + length; 
     if (*s == '-')
     {
-        static constexpr T min_value = (type_traits::integer_limits<T>::lowest)();
+        static constexpr T min_value = (extension_traits::integer_limits<T>::lowest)();
         static constexpr T min_value_div_10 = min_value / 10;
         ++s;
         for (; s < end; ++s)
@@ -755,7 +755,7 @@ to_integer_unchecked(const CharT* s, std::size_t length, T& n)
     }
     else
     {
-        static constexpr T max_value = (type_traits::integer_limits<T>::max)();
+        static constexpr T max_value = (extension_traits::integer_limits<T>::max)();
         static constexpr T max_value_div_10 = max_value / 10;
         for (; s < end; ++s)
         {
@@ -780,10 +780,10 @@ to_integer_unchecked(const CharT* s, std::size_t length, T& n)
 // base16_to_integer
 
 template <class T, class CharT>
-typename std::enable_if<type_traits::integer_limits<T>::is_specialized && type_traits::integer_limits<T>::is_signed,to_integer_result<T,CharT>>::type
+typename std::enable_if<extension_traits::integer_limits<T>::is_specialized && extension_traits::integer_limits<T>::is_signed,to_integer_result<T,CharT>>::type
 base16_to_integer(const CharT* s, std::size_t length, T& n)
 {
-    static_assert(type_traits::integer_limits<T>::is_specialized, "Integer type not specialized");
+    static_assert(extension_traits::integer_limits<T>::is_specialized, "Integer type not specialized");
     JSONCONS_ASSERT(length > 0);
 
     n = 0;
@@ -791,7 +791,7 @@ base16_to_integer(const CharT* s, std::size_t length, T& n)
     const CharT* end = s + length; 
     if (*s == '-')
     {
-        static constexpr T min_value = (type_traits::integer_limits<T>::lowest)();
+        static constexpr T min_value = (extension_traits::integer_limits<T>::lowest)();
         static constexpr T min_value_div_16 = min_value / 16;
         ++s;
         for (; s < end; ++s)
@@ -826,7 +826,7 @@ base16_to_integer(const CharT* s, std::size_t length, T& n)
     }
     else
     {
-        static constexpr T max_value = (type_traits::integer_limits<T>::max)();
+        static constexpr T max_value = (extension_traits::integer_limits<T>::max)();
         static constexpr T max_value_div_16 = max_value / 16;
         for (; s < end; ++s)
         {
@@ -864,16 +864,16 @@ base16_to_integer(const CharT* s, std::size_t length, T& n)
 }
 
 template <class T, class CharT>
-typename std::enable_if<type_traits::integer_limits<T>::is_specialized && !type_traits::integer_limits<T>::is_signed,to_integer_result<T,CharT>>::type
+typename std::enable_if<extension_traits::integer_limits<T>::is_specialized && !extension_traits::integer_limits<T>::is_signed,to_integer_result<T,CharT>>::type
 base16_to_integer(const CharT* s, std::size_t length, T& n)
 {
-    static_assert(type_traits::integer_limits<T>::is_specialized, "Integer type not specialized");
+    static_assert(extension_traits::integer_limits<T>::is_specialized, "Integer type not specialized");
     JSONCONS_ASSERT(length > 0);
 
     n = 0;
     const CharT* end = s + length; 
 
-    static constexpr T max_value = (type_traits::integer_limits<T>::max)();
+    static constexpr T max_value = (extension_traits::integer_limits<T>::max)();
     static constexpr T max_value_div_16 = max_value / 16;
     for (; s < end; ++s)
     {
@@ -909,28 +909,72 @@ base16_to_integer(const CharT* s, std::size_t length, T& n)
     return to_integer_result<T,CharT>(s, to_integer_errc());
 }
 
-#if defined(JSONCONS_HAS_MSC_STRTOD_L)
 
-class to_double_t
+#if defined(JSONCONS_HAS_STD_FROM_CHARS) && JSONCONS_HAS_STD_FROM_CHARS
+
+class chars_to
+{
+public:
+
+    char get_decimal_point() const
+    {
+        return '.';
+    }
+
+    template <class CharT>
+    typename std::enable_if<std::is_same<CharT,char>::value,double>::type
+    operator()(const CharT* s, std::size_t len) const
+    {
+        double val = 0;
+        const auto res = std::from_chars(s, s+len, val);
+        if (res.ec != std::errc())
+        {
+            JSONCONS_THROW(json_runtime_error<std::invalid_argument>("Convert chars to double failed"));
+        }
+        return val;
+    }
+
+    template <class CharT>
+    typename std::enable_if<std::is_same<CharT,wchar_t>::value,double>::type
+    operator()(const CharT* s, std::size_t len) const
+    {
+        std::string input(len,'0');
+        for (size_t i = 0; i < len; ++i)
+        {
+            input[i] = static_cast<char>(s[i]);
+        }
+
+        double val = 0;
+        const auto res = std::from_chars(input.data(), input.data() + len, val);
+        if (res.ec != std::errc())
+        {
+            JSONCONS_THROW(json_runtime_error<std::invalid_argument>("Convert chars to double failed"));
+        }
+        return val;
+    }
+};
+#elif defined(JSONCONS_HAS_MSC_STRTOD_L)
+
+class chars_to
 {
 private:
     _locale_t locale_;
 public:
-    to_double_t()
+    chars_to()
     {
         locale_ = _create_locale(LC_NUMERIC, "C");
     }
-    ~to_double_t() noexcept
+    ~chars_to() noexcept
     {
         _free_locale(locale_);
     }
 
-    to_double_t(const to_double_t&)
+    chars_to(const chars_to&)
     {
         locale_ = _create_locale(LC_NUMERIC, "C");
     }
 
-    to_double_t& operator=(const to_double_t&) 
+    chars_to& operator=(const chars_to&) 
     {
         // Don't assign locale
         return *this;
@@ -970,26 +1014,26 @@ public:
 
 #elif defined(JSONCONS_HAS_STRTOLD_L)
 
-class to_double_t
+class chars_to
 {
 private:
     locale_t locale_;
 public:
-    to_double_t()
+    chars_to()
     {
         locale_ = newlocale(LC_ALL_MASK, "C", (locale_t) 0);
     }
-    ~to_double_t() noexcept
+    ~chars_to() noexcept
     {
         freelocale(locale_);
     }
 
-    to_double_t(const to_double_t&)
+    chars_to(const chars_to&)
     {
         locale_ = newlocale(LC_ALL_MASK, "C", (locale_t) 0);
     }
 
-    to_double_t& operator=(const to_double_t&) 
+    chars_to& operator=(const chars_to&) 
     {
         return *this;
     }
@@ -1027,13 +1071,13 @@ public:
 };
 
 #else
-class to_double_t
+class chars_to
 {
 private:
     std::vector<char> buffer_;
     char decimal_point_;
 public:
-    to_double_t()
+    chars_to()
         : buffer_()
     {
         struct lconv * lc = localeconv();
@@ -1048,8 +1092,8 @@ public:
         buffer_.reserve(100);
     }
 
-    to_double_t(const to_double_t&) = default;
-    to_double_t& operator=(const to_double_t&) = default;
+    chars_to(const chars_to&) = default;
+    chars_to& operator=(const chars_to&) = default;
 
     char get_decimal_point() const
     {
