@@ -1,4 +1,4 @@
-// Copyright 2013 Daniel Parker
+// Copyright 2013-2023 Daniel Parker
 // Distributed under the Boost license, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
@@ -10,8 +10,11 @@
 #include <string>
 #include <limits> // std::numeric_limits
 #include <cwchar>
+#include <functional>
 #include <jsoncons/json_exception.hpp>
-#include <jsoncons/more_type_traits.hpp>
+#include <jsoncons/json_error.hpp>
+#include <jsoncons/extension_traits.hpp>
+#include <jsoncons/ser_context.hpp>
 
 namespace jsoncons {
 
@@ -23,6 +26,9 @@ JSONCONS_DEPRECATED_MSG("Instead, use float_chars_format") typedef float_chars_f
 
 enum class indenting : uint8_t {no_indent = 0, indent = 1};
 
+// legacy
+using indenting = indenting;
+
 enum class line_split_kind  : uint8_t {same_line,new_line,multi_line};
 
 enum class bigint_chars_format : uint8_t {number, base10, base64, base64url
@@ -40,6 +46,36 @@ enum class byte_string_chars_format : uint8_t {none=0,base16,base64,base64url};
 
 enum class spaces_option : uint8_t {no_spaces=0,space_after,space_before,space_before_and_after};
 
+
+struct default_json_parsing
+{
+    bool operator()(json_errc ec, const ser_context&) noexcept 
+    {
+        return ec == json_errc::illegal_comment;
+    }
+};
+
+struct strict_json_parsing
+{
+    bool operator()(json_errc, const ser_context&) noexcept
+    {
+        return false;
+    }
+};
+
+struct allow_trailing_commas
+{
+    bool operator()(const std::error_code& ec, const ser_context&) noexcept 
+    {
+        return ec == json_errc::illegal_comment || ec == jsoncons::json_errc::extra_comma;
+    }
+};
+
+#if !defined(JSONCONS_NO_DEPRECATED)
+JSONCONS_DEPRECATED_MSG("Instead, use default_json_parsing") typedef default_json_parsing default_parse_error_handler;
+JSONCONS_DEPRECATED_MSG("Instead, use strict_json_parsing") typedef strict_json_parsing strict_parse_error_handler;
+#endif
+
 template <class CharT>
 class basic_json_options;
 
@@ -324,18 +360,18 @@ public:
     using typename super_type::char_type;
     using typename super_type::string_type;
 private:
-    bool lossless_number_:1;
+    bool lossless_number_;
+    std::function<bool(json_errc,const ser_context&)> err_handler_;
 public:
     basic_json_decode_options()
-        : lossless_number_(false)
+        : lossless_number_(false), err_handler_(default_json_parsing())
     {
     }
 
     basic_json_decode_options(const basic_json_decode_options&) = default;
 
     basic_json_decode_options(basic_json_decode_options&& other)
-        : super_type(std::forward<basic_json_decode_options>(other)),
-                     lossless_number_(other.lossless_number_)
+        : super_type(std::move(other)), lossless_number_(other.lossless_number_), err_handler_(std::move(other.err_handler_))
     {
     }
 
@@ -346,6 +382,11 @@ public:
         return lossless_number_;
     }
 
+    const std::function<bool(json_errc,const ser_context&)>& err_handler() const 
+    {
+        return err_handler_;
+    }
+
 #if !defined(JSONCONS_NO_DEPRECATED)
     JSONCONS_DEPRECATED_MSG("Instead, use lossless_number()")
     bool dec_to_str() const 
@@ -409,7 +450,7 @@ public:
     basic_json_encode_options(const basic_json_encode_options&) = default;
 
     basic_json_encode_options(basic_json_encode_options&& other)
-        : super_type(std::forward<basic_json_encode_options>(other)),
+        : super_type(std::move(other)),
           escape_all_non_ascii_(other.escape_all_non_ascii_),
           escape_solidus_(other.escape_solidus_),
           pad_inside_object_braces_(other.pad_inside_object_braces_),
@@ -544,6 +585,7 @@ public:
     using basic_json_decode_options<CharT>::neginf_to_num;
 
     using basic_json_decode_options<CharT>::lossless_number;
+    using basic_json_decode_options<CharT>::err_handler;
 
     using basic_json_encode_options<CharT>::byte_string_format;
     using basic_json_encode_options<CharT>::bigint_format;
@@ -608,7 +650,7 @@ public:
     basic_json_options& inf_to_str(const string_type& value, bool enable_inverse = true)
     {
         this->enable_inf_to_str_ = true;
-        this->enable_inf_to_str_ = enable_inverse;
+        this->enable_str_to_inf_ = enable_inverse;
         this->inf_to_num_.clear();
         this->inf_to_str_ = value;
         return *this;
@@ -617,7 +659,7 @@ public:
     basic_json_options& neginf_to_str(const string_type& value, bool enable_inverse = true)
     {
         this->enable_neginf_to_str_ = true;
-        this->enable_neginf_to_str_ = enable_inverse;
+        this->enable_str_to_neginf_ = enable_inverse;
         this->neginf_to_num_.clear();
         this->neginf_to_str_ = value;
         return *this;
@@ -725,6 +767,12 @@ public:
         return *this;
     }
 
+    basic_json_options& err_handler(const std::function<bool(json_errc,const ser_context&)>& value) 
+    {
+        this->err_handler_ = value;
+        return *this;
+    }
+
     basic_json_options& line_length_limit(std::size_t value)
     {
         this->line_length_limit_ = value;
@@ -821,8 +869,6 @@ private:
                     state = input_state::begin_quote;
                     break;
                 case input_state::begin_quote:
-                    state = input_state::end_quote;
-                    break;
                 case input_state::character:
                     state = input_state::end_quote;
                     break;
